@@ -67,11 +67,11 @@ zenity_colorselection (ZenityData *data, ZenityColorData *color_data)
 
 	zenity_util_show_dialog (dialog);
 
-	if (data->timeout_delay > 0) {
-		g_timeout_add_seconds (data->timeout_delay,
-			(GSourceFunc) zenity_util_timeout_handle,
-			dialog);
+	if (data->timeout_delay > 0)
+	{
+		ZENITY_UTIL_SETUP_TIMEOUT (dialog)
 	}
+
 	zenity_util_gapp_main (GTK_WINDOW(dialog));
 }
 
@@ -83,7 +83,7 @@ zenity_colorselection_dialog_response (GtkWidget *widget, int response, gpointer
 	switch (response)
 	{
 		case GTK_RESPONSE_OK:
-			zenity_util_exit_code_with_data (ZENITY_OK, zen_data);
+			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
 			gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (widget), &color);
 			g_print ("%s\n", gdk_rgba_to_string (&color));
 			break;
@@ -97,10 +97,7 @@ zenity_colorselection_dialog_response (GtkWidget *widget, int response, gpointer
 			break;
 
 		default:
-			if (zen_data->extra_label &&
-				response < (int)g_strv_length (zen_data->extra_label))
-				printf ("%s\n", zen_data->extra_label[response]);
-			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
+			ZENITY_UTIL_RESPONSE_HANDLE_EXTRA_BUTTONS
 			break;
 	}
 	zenity_util_gapp_quit (GTK_WINDOW(widget), zen_data);
