@@ -31,6 +31,12 @@
 
 #include <config.h>
 
+/* TODO: Many items in this source file need to be ported away from deprecated
+ * API in GTK 4.10+. Suppress the whirlwind of warnings for now since we're
+ * well-aware of this.
+ */
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 static ZenityData *zen_data;
 static GSList *selected;
 static void zenity_forms_dialog_response (GtkWidget *widget, char *rstr, gpointer data);
@@ -59,11 +65,7 @@ static GtkWidget *
 zenity_forms_create_and_fill_combo (ZenityFormsData *forms_data,
 		int combo_number)
 {
-	g_autoptr(GtkListStore) list_store = NULL;
-	GtkWidget *combo_box;
-	GtkCellRenderer *renderer;
-
-	list_store = gtk_list_store_new (1, G_TYPE_STRING);
+	GtkWidget *dropdown = NULL;
 
 	if (forms_data->combo_values)
 	{
@@ -72,36 +74,14 @@ zenity_forms_create_and_fill_combo (ZenityFormsData *forms_data,
 
 		if (combo_values)
 		{
-			char **row_values = g_strsplit_set (combo_values, "|", -1);
+			g_auto(GStrv) row_values = g_strsplit_set (combo_values, "|", -1);
 
 			if (row_values)
-			{
-				int i = 0;
-				GtkTreeIter iter;
-				char *row = row_values[i];
-
-				while (row != NULL)
-				{
-					gtk_list_store_append (list_store, &iter);
-					gtk_list_store_set (list_store, &iter, 0, row, -1);
-					row = row_values[++i];
-				}
-				g_strfreev (row_values);
-			}
+				dropdown = gtk_drop_down_new_from_strings ((const char* const*)row_values);
 		}
 	}
 
-	combo_box = gtk_combo_box_new_with_model (GTK_TREE_MODEL(list_store));
-
-	renderer = gtk_cell_renderer_text_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT(combo_box),
-			renderer,
-			TRUE);	/* gboolean expand */
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT(combo_box), renderer,
-			"text", 0,
-			NULL);
-
-	return combo_box;
+	return dropdown;
 }
 
 static GtkWidget *
@@ -345,10 +325,9 @@ zenity_forms_dialog (ZenityData *data, ZenityFormsData *forms_data)
 
 	if (data->timeout_delay > 0)
 	{
-		g_timeout_add_seconds (data->timeout_delay,
-			(GSourceFunc) zenity_util_timeout_handle,
-			dialog);
+		ZENITY_UTIL_SETUP_TIMEOUT (dialog)
 	}
+
 	zenity_util_gapp_main (GTK_WINDOW(dialog));
 }
 
@@ -359,10 +338,7 @@ zenity_forms_dialog_output (ZenityFormsData *forms_data)
 	guint day, year, month;
 	g_autoptr(GDate) date = NULL;
 	char time_string[128];
-	g_autofree char *combo_value = NULL;
 	GtkTreeSelection *selection;
-	g_autoptr(GtkListStore) list_store = NULL;
-	GtkTreeIter iter;
 
 	for (tmp = forms_data->list; tmp; tmp = tmp->next)
 	{
@@ -411,23 +387,14 @@ zenity_forms_dialog_output (ZenityFormsData *forms_data)
 				break;
 
 			case ZENITY_FORMS_COMBO:
-				if (gtk_combo_box_get_active_iter
-						(GTK_COMBO_BOX(zenity_value->forms_widget), &iter))
-				{
-					list_store =
-						GTK_LIST_STORE(gtk_combo_box_get_model
-								(GTK_COMBO_BOX(zenity_value->forms_widget)));
+			{
+				GtkStringObject *strobj = gtk_drop_down_get_selected_item (GTK_DROP_DOWN(zenity_value->forms_widget));
 
-					gtk_tree_model_get (GTK_TREE_MODEL(list_store),
-						&iter,
-						0,
-						&combo_value,
-						-1);
-
-					g_print ("%s", combo_value);
-				}
+				if (strobj)
+					g_print ("%s", gtk_string_object_get_string (strobj));
 				else
 					g_print (" ");
+			}
 				break;
 
 		}
@@ -441,13 +408,13 @@ static void
 zenity_forms_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 {
 	ZenityFormsData *forms_data = data;
-	ZenityExitCode response = zenity_util_parse_dialog_response (rstr);
+	int response = zenity_util_parse_dialog_response (rstr);
 
 	switch (response)
 	{
 		case ZENITY_OK:
 			zenity_forms_dialog_output (forms_data);
-			zenity_util_exit_code_with_data (ZENITY_OK, zen_data);
+			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
 			break;
 
 		case ZENITY_CANCEL:
@@ -464,11 +431,10 @@ zenity_forms_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 			break;
 
 		default:
-			if (zen_data->extra_label &&
-				response < (int)g_strv_length (zen_data->extra_label))
-				printf ("%s\n", zen_data->extra_label[response]);
-			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
+			ZENITY_UTIL_RESPONSE_HANDLE_EXTRA_BUTTONS
 			break;
 	}
 	zenity_util_gapp_quit (GTK_WINDOW(widget), zen_data);
 }
+
+G_GNUC_END_IGNORE_DEPRECATIONS
