@@ -199,7 +199,6 @@ zenity_progress_handle_stdin (GIOChannel *source, GIOCondition condition,
 			else if (g_str_has_prefix (string->str, "pulsate"))
 			{
 				char *colon, *value;
-				g_autofree char *command = NULL;
 
 				zenity_util_strip_newline (string->str);
 
@@ -208,10 +207,6 @@ zenity_progress_handle_stdin (GIOChannel *source, GIOCondition condition,
 					continue;
 				}
 
-				/* split off the command and value */
-				command =
-					g_strstrip (g_strndup (string->str, colon - string->str));
-
 				value = colon + 1;
 				while (*value && g_ascii_isspace (*value))
 					value++;
@@ -398,7 +393,26 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
 	if (!auto_close)
 	{
 		adw_message_dialog_add_response (ADW_MESSAGE_DIALOG(dialog), "ok", _("_OK"));
-		adw_message_dialog_set_response_enabled (ADW_MESSAGE_DIALOG(dialog), "ok", FALSE);
+		if (progress_data->percentage == 100) {
+			adw_message_dialog_set_response_enabled (ADW_MESSAGE_DIALOG(dialog), "ok", TRUE);
+			adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG(dialog), "ok");
+		} else {
+			adw_message_dialog_set_response_enabled (ADW_MESSAGE_DIALOG(dialog), "ok", FALSE);
+		}
+	}
+	else
+	{
+		/* If the user specifies percentage should be 100 and the dialog should
+		 * auto-close, this is completely pointless and we print an error.
+		 */
+		if (progress_data->percentage == 100) {
+			/* Translators: do not translate tokens starting with '--'; these
+			 * are command-line options which are not translatable.
+			 */
+			g_printerr (_("Combining the options --auto-close and --percentage=100 is not supported.\n"));
+			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ERROR);
+			zenity_util_gapp_quit (NULL, zen_data);
+		}
 	}
 
 	zenity_util_show_dialog (dialog);
@@ -406,22 +420,21 @@ zenity_progress (ZenityData *data, ZenityProgressData *progress_data)
 
 	if (data->timeout_delay > 0)
 	{
-		g_timeout_add_seconds (data->timeout_delay,
-			(GSourceFunc) zenity_util_timeout_handle,
-			NULL);
+		ZENITY_UTIL_SETUP_TIMEOUT (dialog)
 	}
+
 	zenity_util_gapp_main (GTK_WINDOW(dialog));
 }
 
 static void
 zenity_progress_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 {
-	ZenityExitCode response = zenity_util_parse_dialog_response (rstr);
+	int response = zenity_util_parse_dialog_response (rstr);
 
 	switch (response)
 	{
 		case ZENITY_OK:
-			zenity_util_exit_code_with_data (ZENITY_OK, zen_data);
+			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
 			break;
 
 		case ZENITY_CANCEL:
@@ -436,7 +449,7 @@ zenity_progress_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 			break;
 
 		case ZENITY_TIMEOUT:
-			zenity_util_exit_code_with_data (ZENITY_TIMEOUT, zen_data);
+			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_TIMEOUT);
 			break;
 
 		case ZENITY_ESC:
@@ -444,12 +457,7 @@ zenity_progress_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 			break;
 
 		default:
-			if (zen_data->extra_label &&
-				response < (int)g_strv_length (zen_data->extra_label))
-			{
-				printf ("%s\n", zen_data->extra_label[response]);
-			}
-			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
+			ZENITY_UTIL_RESPONSE_HANDLE_EXTRA_BUTTONS
 			break;
 	}
 	zenity_util_gapp_quit (GTK_WINDOW(widget), zen_data);
