@@ -125,9 +125,7 @@ zenity_scale (ZenityData *data, ZenityScaleData *scale_data)
 
 	if (data->timeout_delay > 0)
 	{
-		g_timeout_add_seconds (data->timeout_delay,
-			(GSourceFunc) zenity_util_timeout_handle,
-			dialog);
+		ZENITY_UTIL_SETUP_TIMEOUT (dialog)
 	}
 
 	zenity_util_gapp_main (GTK_WINDOW(dialog));
@@ -143,12 +141,12 @@ static void
 zenity_scale_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 {
 	ZenityData *zen_data = data;
-	ZenityExitCode response = zenity_util_parse_dialog_response (rstr);
+	int response = zenity_util_parse_dialog_response (rstr);
 
 	switch (response)
 	{
 		case ZENITY_OK:
-			zenity_util_exit_code_with_data (ZENITY_OK, zen_data);
+			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
 			g_print ("%.0f\n", gtk_range_get_value (GTK_RANGE (scale)));
 			break;
 
@@ -166,12 +164,7 @@ zenity_scale_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 			break;
 
 		default:
-			if (zen_data->extra_label &&
-				response < (int)g_strv_length (zen_data->extra_label))
-			{
-				printf ("%s\n", zen_data->extra_label[response]);
-			}
-			zen_data->exit_code = zenity_util_return_exit_code (ZENITY_ESC);
+			ZENITY_UTIL_RESPONSE_HANDLE_EXTRA_BUTTONS
 			break;
 	}
 
