@@ -68,14 +68,12 @@ zenity_configure_webkit (WebKitWebView *web_view) {
 	g_object_set (G_OBJECT (settings), "enable-html5-database", FALSE, NULL);
 	g_object_set (
 		G_OBJECT (settings), "enable-html5-local-storage", FALSE, NULL);
-	g_object_set (G_OBJECT (settings), "enable-java", FALSE, NULL);
 	g_object_set (G_OBJECT (settings), "enable-javascript", FALSE, NULL);
 	g_object_set (G_OBJECT (settings),
 		"enable-offline-web-application-cache",
 		FALSE,
 		NULL);
 	g_object_set (G_OBJECT (settings), "enable-page-cache", FALSE, NULL);
-	g_object_set (G_OBJECT (settings), "enable-plugins", FALSE, NULL);
 	/*
 	  Stick to defaults
 	  "enforce-96-dpi"           gboolean              : Read / Write /
@@ -158,7 +156,7 @@ zenity_text_handle_stdin (GIOChannel *channel, GIOCondition condition,
 #define BUF_SIZE 1024
 	static GtkTextBuffer *buffer;
 	static GtkTextView *text_view;
-	char buf[BUF_SIZE];
+	char buf[BUF_SIZE + 1];
 
 	gsize len;
 
@@ -215,6 +213,7 @@ zenity_text_handle_stdin (GIOChannel *channel, GIOCondition condition,
 			}
 			else
 			{
+				buf[len] = '\0';
 				gtk_text_buffer_insert (buffer, &end, buf, len);
 			}
 
@@ -304,12 +303,15 @@ zenity_text (ZenityData *data, ZenityTextData *text_data)
 		css_str = zenity_util_pango_font_description_to_css (desc);
 			
 		provider = gtk_css_provider_new ();
+
+		G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 		gtk_css_provider_load_from_data (provider, css_str, -1);
 
 		context = gtk_widget_get_style_context (GTK_WIDGET(text_view));
 		gtk_style_context_add_provider (context,
 				GTK_STYLE_PROVIDER(provider),
 				GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+		G_GNUC_END_IGNORE_DEPRECATIONS
 	}
 
 	if (text_data->uri)
@@ -361,10 +363,10 @@ zenity_text (ZenityData *data, ZenityTextData *text_data)
 		g_autoptr(WebKitWebContext) wk_context = webkit_web_context_new ();
 		g_autoptr(WebKitNetworkSession) wk_session = webkit_network_session_new_ephemeral ();
 
-		web_kit = WEBKIT_WEB_VIEW (g_object_new (WEBKIT_TYPE_WEB_VIEW,
+		web_kit = g_object_new (WEBKIT_TYPE_WEB_VIEW,
 				"web-context", wk_context,
 				"network-session", wk_session,
-				NULL));
+				NULL);
 		scrolled_window = GTK_WIDGET (
 			gtk_builder_get_object (builder, "zenity_text_scrolled_window"));
 
@@ -409,10 +411,9 @@ zenity_text (ZenityData *data, ZenityTextData *text_data)
 
 	if (data->timeout_delay > 0)
 	{
-		g_timeout_add_seconds (data->timeout_delay,
-			(GSourceFunc) zenity_util_timeout_handle,
-			dialog);
+		ZENITY_UTIL_SETUP_TIMEOUT (dialog)
 	}
+
 	zenity_util_gapp_main (GTK_WINDOW(dialog));
 }
 
@@ -441,7 +442,7 @@ static void
 zenity_text_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 {
 	ZenityData *zen_data = data;
-	ZenityExitCode response = zenity_util_parse_dialog_response (rstr);
+	int response = zenity_util_parse_dialog_response (rstr);
 
 	switch (response)
 	{
@@ -464,12 +465,7 @@ zenity_text_dialog_response (GtkWidget *widget, char *rstr, gpointer data)
 			break;
 			
 		default:
-			if (zen_data->extra_label &&
-				response < (int)g_strv_length (zen_data->extra_label))
-			{
-				printf ("%s\n", zen_data->extra_label[response]);
-			}
-			zenity_util_exit_code_with_data (ZENITY_ESC, zen_data);
+			ZENITY_UTIL_RESPONSE_HANDLE_EXTRA_BUTTONS
 			break;
 	}
 	zenity_util_gapp_quit (GTK_WINDOW(widget), zen_data);
