@@ -46,7 +46,20 @@
 #define ZENITY_CANCEL_DEFAULT 1
 #define ZENITY_ESC_DEFAULT 1
 #define ZENITY_ERROR_DEFAULT -1
-#define ZENITY_EXTRA_DEFAULT 127
+
+/* This is an odd exit code for extra labels, but it is maintained
+ * for backwards compatibility with zenity <= 3.x.
+ */
+#define ZENITY_EXTRA_DEFAULT 1
+
+/* This exit code number is arbitrary, but since for the entire 3.x release
+ * cycle, zenity would essentially exit(ZENITY_TIMEOUT), which happened to be
+ * defined as 5 based on where it was placed in the enum sequence. So
+ * hardcoding it as 5 now in case any pre-existing scripts relied upon that
+ * being the exit status for timeouts.
+ */
+#define ZENITY_TIMEOUT_DEFAULT 5
+
 #define ZENITY_UI_RESOURCE_PATH RESOURCE_BASE_PATH "/zenity.ui"
 
 GIcon *
@@ -267,7 +280,7 @@ zenity_util_return_exit_code (ZenityExitCode value)
 			if (!env_var)
 				env_var = g_getenv ("DIALOG_TIMEOUT");
 			if (!env_var)
-				retval = ZENITY_TIMEOUT;
+				retval = ZENITY_TIMEOUT_DEFAULT;
 			break;
 
 		default:
@@ -279,12 +292,6 @@ zenity_util_return_exit_code (ZenityExitCode value)
 	return retval;
 }
 
-void
-zenity_util_exit_code_with_data (ZenityExitCode value, ZenityData *zen_data)
-{
-	zen_data->exit_code = zenity_util_return_exit_code (value);
-}
-
 /* This function was written by Matthias Clasen and is included somewhere in
  * the GTK source tree.. I believe it is also included in libdazzle, but I
  * didn't want to include a whole dependency just for one function. LGPL, but
@@ -417,17 +424,17 @@ zenity_util_pango_font_description_to_css (PangoFontDescription *desc)
 void
 zenity_util_show_dialog (GtkWidget *dialog)
 {
-	gtk_widget_show (dialog);
+	gtk_window_present (GTK_WINDOW(dialog));
 }
 
 gboolean
 zenity_util_timeout_handle (AdwMessageDialog *dialog)
 {
-	if (dialog) {
+	if (dialog && ADW_IS_MESSAGE_DIALOG (dialog)) {
 		adw_message_dialog_response (dialog, "timeout");
 	}
 	else {
-		exit (ZENITY_TIMEOUT);
+		exit (zenity_util_return_exit_code (ZENITY_TIMEOUT));
 	}
 	return FALSE;
 }
@@ -469,7 +476,7 @@ zenity_util_gapp_quit (GtkWindow *window, ZenityData *data)
 		exit (data->exit_code);
 
 	if (window)
-		gtk_window_destroy (window);
+		gtk_application_remove_window (GTK_APPLICATION(g_application_get_default ()), window);
 	else
 		g_application_release (g_application_get_default ());
 
@@ -493,11 +500,20 @@ zenity_util_parse_dialog_response (const char *response)
 	}
 	else if (response[0] >= '0' && response[0] <= '9')
 	{
-		/* FIXME - atoi returns 0 on error, so this *could* mean the function
-		 * failed - but that would be a programmer error, so we'll leave this
-		 * in place.
+		/* Since atoi can return 0 upon an error or a valid "0", deal with it
+		 * as a special case here.
 		 */
-		return atoi (response);
+		if (g_strcmp0 (response, "0") == 0)
+			return 0;
+		else
+		{
+			int retval = atoi (response);
+
+			if (G_UNLIKELY (! retval))
+				g_error ("%s: Programmer error: invalid integer string", __func__);
+
+			return retval;
+		}
 	}
 	else
 	{
