@@ -49,12 +49,14 @@ G_BEGIN_DECLS
  * custom equivalent, remove that test here.
  */
 #define ZENITY_UTIL_ADD_EXTRA_LABELS(DIALOG) \
+	G_GNUC_BEGIN_IGNORE_DEPRECATIONS \
 	for (int i = 0; data->extra_label[i] != NULL; ++i) { \
 		if (ADW_IS_MESSAGE_DIALOG (DIALOG)) \
 			zenity_util_add_button (ADW_MESSAGE_DIALOG (DIALOG), data->extra_label[i], i); \
 		else if (GTK_IS_DIALOG (DIALOG)) \
 			gtk_dialog_add_button (GTK_DIALOG (dialog), data->extra_label[i], i); \
-	}
+	} \
+	G_GNUC_END_IGNORE_DEPRECATIONS
 
 #define ZENITY_UTIL_SETUP_OK_BUTTON_LABEL(DIALOG) \
 	if (adw_message_dialog_has_response (ADW_MESSAGE_DIALOG(DIALOG), "ok")) \
@@ -69,6 +71,21 @@ G_BEGIN_DECLS
 	else if (adw_message_dialog_has_response (ADW_MESSAGE_DIALOG(DIALOG), "no")) \
 		adw_message_dialog_set_response_label (ADW_MESSAGE_DIALOG(DIALOG), "no", data->cancel_label);
 
+#define ZENITY_UTIL_RESPONSE_HANDLE_EXTRA_BUTTONS \
+	if (zen_data->extra_label && \
+		response < (int)g_strv_length (zen_data->extra_label)) \
+	{ \
+		printf ("%s\n", zen_data->extra_label[response]); \
+		zen_data->exit_code = zenity_util_return_exit_code (ZENITY_EXTRA); \
+	} \
+	else \
+		g_assert_not_reached ();
+
+#define ZENITY_UTIL_SETUP_TIMEOUT(DIALOG) \
+	g_timeout_add_seconds (data->timeout_delay, \
+		(GSourceFunc) zenity_util_timeout_handle, \
+		DIALOG);
+
 GIcon *zenity_util_gicon_from_string (const char *str);
 GtkBuilder *zenity_util_load_ui_file (const char *widget_root,
 		...) G_GNUC_NULL_TERMINATED;
@@ -77,13 +94,12 @@ gboolean zenity_util_fill_file_buffer (GtkTextBuffer *buffer,
 		const char *filename);
 void zenity_util_show_help (GError **error);
 int zenity_util_return_exit_code (ZenityExitCode value);
-void zenity_util_exit_code_with_data (ZenityExitCode value, ZenityData *data);
 void zenity_util_show_dialog (GtkWidget *widget);
 gboolean zenity_util_timeout_handle (AdwMessageDialog *dialog);
 char *zenity_util_pango_font_description_to_css (PangoFontDescription *desc);
 void zenity_util_gapp_main (GtkWindow *window);
 void zenity_util_gapp_quit (GtkWindow *window, ZenityData *data);
-ZenityExitCode zenity_util_parse_dialog_response (const char *response);
+int zenity_util_parse_dialog_response (const char *response);
 GtkWidget *zenity_util_add_button (AdwMessageDialog *dialog, const char
 		*button_text, ZenityExitCode response_id);
 
