@@ -122,7 +122,7 @@ static void
 zenity_tree_item_class_init (ZenityTreeItemClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
-	GParamFlags flags = G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT;
+	GParamFlags flags = G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT | G_PARAM_EXPLICIT_NOTIFY;
 
 	object_class->dispose = zenity_tree_item_dispose;
 	object_class->finalize = zenity_tree_item_finalize;
@@ -243,6 +243,7 @@ enum zenity_tree_column_view_prop_enum
 	MULTI = 1,
 	LIST_TYPE,
 	MODEL,
+	HIDE_HEADER,
 	N_PROPERTIES_ZENITY_TREE_COLUMN_VIEW
 };
 
@@ -253,7 +254,9 @@ struct _ZenityTreeColumnView
 	GtkWidget parent_instance;
 	GtkWidget *scrolled_window;
 	GtkColumnView *child_cv;
+	gulong child_cv_activate_handler_id;
 	gboolean multi;
+	gboolean hide_header;
 	ZenityTreeListType list_type;
 	GListModel *model;
 	GtkStringFilter *filter;
@@ -324,12 +327,25 @@ zenity_tree_column_view_set_model (ZenityTreeColumnView *self, GListModel *model
 	g_object_notify_by_pspec (G_OBJECT(self), zenity_tree_column_view_properties[MODEL]);
 }
 
+/* Returns the raw GListModel of *all* items within the column view. This will
+ * *not* account for filtered items by search or what-have-you.
+ */
 GListModel *
 zenity_tree_column_view_get_model (ZenityTreeColumnView *self)
 {
 	return self->model;
 }
 
+/* Returns the actual selection model of the column view, which, as opposed to
+ * the raw list model, may be filtered to show what is actually visible within
+ * the widget.
+ */
+GtkSelectionModel *
+zenity_tree_column_view_get_selection_model (ZenityTreeColumnView *self)
+{
+	return gtk_column_view_get_model (self->child_cv);
+}
+
 static void
 zenity_tree_column_view_emit_activated (ZenityTreeColumnView *self)
 {
@@ -339,8 +355,7 @@ zenity_tree_column_view_emit_activated (ZenityTreeColumnView *self)
 static void
 cv_check_or_radio_activated_cb (ZenityTreeColumnView *self, guint position, GtkColumnView *cv)
 {
-	GListModel *model = zenity_tree_column_view_get_model (self);
-	ZenityTreeRow *row = g_list_model_get_item (model, position);
+	ZenityTreeRow *row = g_list_model_get_item (self->model, position);
 	ZenityTreeItem *item = zenity_tree_row_get_item (row, 0);
 	GtkWidget *item_child = zenity_tree_item_get_child (item);
 	GtkCheckButton *cb;
@@ -366,25 +381,39 @@ cv_check_or_radio_activated_cb (ZenityTreeColumnView *self, guint position, GtkC
 	}
 }
 
+void
+zenity_tree_column_view_set_multi (ZenityTreeColumnView *self, gboolean multi)
+{
+	self->multi = multi;
+	g_object_notify_by_pspec (G_OBJECT(self), zenity_tree_column_view_properties[MULTI]);
+}
+
+void
+zenity_tree_column_view_set_hide_header (ZenityTreeColumnView *self, gboolean hide)
+{
+	self->hide_header = hide;
+	g_object_notify_by_pspec (G_OBJECT(self), zenity_tree_column_view_properties[HIDE_HEADER]);
+}
+
 void
 zenity_tree_column_view_set_list_type (ZenityTreeColumnView *self, ZenityTreeListType type)
 {
 	self->list_type = type;
 
-	g_signal_handlers_disconnect_by_func (self->child_cv, zenity_tree_column_view_emit_activated, self);
+	g_clear_signal_handler (&self->child_cv_activate_handler_id, self->child_cv);
 
 	switch (self->list_type)
 	{
 		case ZENITY_TREE_LIST_NONE:
 		case ZENITY_TREE_LIST_IMAGE:
 			gtk_column_view_set_single_click_activate (self->child_cv, FALSE);
-			g_signal_connect_swapped (self->child_cv, "activate", G_CALLBACK(zenity_tree_column_view_emit_activated), self);
+			self->child_cv_activate_handler_id = g_signal_connect_object (self->child_cv, "activate", G_CALLBACK(zenity_tree_column_view_emit_activated), self, G_CONNECT_SWAPPED);
 			break;
 
 		case ZENITY_TREE_LIST_RADIO:
 		case ZENITY_TREE_LIST_CHECK:
 			gtk_column_view_set_single_click_activate (self->child_cv, TRUE);
-			g_signal_connect_swapped (self->child_cv, "activate", G_CALLBACK(cv_check_or_radio_activated_cb), self);
+			self->child_cv_activate_handler_id = g_signal_connect_object (self->child_cv, "activate", G_CALLBACK(cv_check_or_radio_activated_cb), self, G_CONNECT_SWAPPED);
 			break;
 
 		default:
@@ -407,6 +436,12 @@ zenity_tree_column_view_get_multi (ZenityTreeColumnView *self)
 	return self->multi;
 }
 
+gboolean
+zenity_tree_column_view_get_hide_header (ZenityTreeColumnView *self)
+{
+	return self->hide_header;
+}
+
 static void
 zenity_tree_column_view_set_property (GObject *object,
 		guint property_id,
@@ -418,7 +453,7 @@ zenity_tree_column_view_set_property (GObject *object,
 	switch (property_id)
 	{
 		case MULTI:
-			self->multi = g_value_get_boolean (value);
+			zenity_tree_column_view_set_multi (self, g_value_get_boolean (value));
 			break;
 
 		case LIST_TYPE:
@@ -429,6 +464,10 @@ zenity_tree_column_view_set_property (GObject *object,
 			zenity_tree_column_view_set_model (self, G_LIST_MODEL(g_value_get_object (value)));
 			break;
 
+		case HIDE_HEADER:
+			zenity_tree_column_view_set_hide_header (self, g_value_get_boolean (value));
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 			break;
@@ -446,7 +485,7 @@ zenity_tree_column_view_get_property (GObject *object,
 	switch (property_id)
 	{
 		case MULTI:
-			g_value_set_boolean (value, self->multi);
+			g_value_set_boolean (value, zenity_tree_column_view_get_multi (self));
 			break;
 
 		case LIST_TYPE:
@@ -457,6 +496,10 @@ zenity_tree_column_view_get_property (GObject *object,
 			g_value_set_object (value, G_OBJECT(zenity_tree_column_view_get_model (self)));
 			break;
 
+		case HIDE_HEADER:
+			g_value_set_boolean (value, zenity_tree_column_view_get_hide_header (self));
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 			break;
@@ -468,6 +511,7 @@ zenity_tree_column_view_dispose (GObject *object)
 {
 	ZenityTreeColumnView *self = ZENITY_TREE_COLUMN_VIEW(object);
 
+	g_clear_signal_handler (&self->child_cv_activate_handler_id, self->child_cv);
 	g_clear_pointer (&self->scrolled_window, gtk_widget_unparent);
 
 	G_OBJECT_CLASS(zenity_tree_column_view_parent_class)->dispose (object);
@@ -489,7 +533,7 @@ zenity_tree_column_view_class_init (ZenityTreeColumnViewClass *klass)
 {
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
-	GParamFlags flags = G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT;
+	GParamFlags flags = G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT | G_PARAM_EXPLICIT_NOTIFY;
 
 	object_class->dispose = zenity_tree_column_view_dispose;
 	object_class->set_property = zenity_tree_column_view_set_property;
@@ -501,6 +545,8 @@ zenity_tree_column_view_class_init (ZenityTreeColumnViewClass *klass)
 			ZENITY_TYPE_TREE_LIST_TYPE, ZENITY_TREE_LIST_NONE, flags);
 	zenity_tree_column_view_properties[MODEL] = g_param_spec_object ("model", NULL, NULL,
 			G_TYPE_LIST_MODEL, flags);
+	zenity_tree_column_view_properties[HIDE_HEADER] = g_param_spec_boolean ("hide-header", NULL, NULL,
+			TRUE, flags);
 
 	g_object_class_install_properties (object_class, N_PROPERTIES_ZENITY_TREE_COLUMN_VIEW, zenity_tree_column_view_properties);
 
@@ -616,7 +662,7 @@ zenity_tree_column_view_add_column (ZenityTreeColumnView *self, const char *col_
 
 	/* nb: seems the signals for the factory need to be setup first *before* creating the column. */
 
-	column = gtk_column_view_column_new (col_name, factory);
+	column = gtk_column_view_column_new (self->hide_header ? NULL : col_name, factory);
 	
 	if (new_col_index == 0 &&
 			(self->list_type == ZENITY_TREE_LIST_CHECK || self->list_type == ZENITY_TREE_LIST_RADIO))
@@ -635,11 +681,9 @@ zenity_tree_column_view_add_column (ZenityTreeColumnView *self, const char *col_
 void
 zenity_tree_column_view_foreach_item (ZenityTreeColumnView *self, GFunc func, gpointer user_data)
 {
-	GListModel *model = zenity_tree_column_view_get_model (self);
-
-	for (guint i = 0; i < g_list_model_get_n_items (model); ++i)
+	for (guint i = 0; i < g_list_model_get_n_items (self->model); ++i)
 	{
-		ZenityTreeRow *row = g_list_model_get_item (model, i);
+		ZenityTreeRow *row = g_list_model_get_item (self->model, i);
 
 		for (guint j = 0; j < zenity_tree_row_get_n_items (row); ++j)
 		{
@@ -653,11 +697,9 @@ zenity_tree_column_view_foreach_item (ZenityTreeColumnView *self, GFunc func, gp
 void
 zenity_tree_column_view_foreach_row (ZenityTreeColumnView *self, GFunc func, gpointer user_data)
 {
-	GListModel *model = zenity_tree_column_view_get_model (self);
-
-	for (guint i = 0; i < g_list_model_get_n_items (model); ++i)
+	for (guint i = 0; i < g_list_model_get_n_items (self->model); ++i)
 	{
-		ZenityTreeRow *row = g_list_model_get_item (model, i);
+		ZenityTreeRow *row = g_list_model_get_item (self->model, i);
 		func (row, user_data);
 	}
 }
