@@ -512,6 +512,33 @@ create_test ("test_no_markup",
 	}
 );
 
+create_test ("issue_75_ensure_num_entries",
+	"3 entry dialogs will appear. For #1, just click OK.\n\tFor #2, select the 2nd item and click OK.\n\tFor #3, select the 3rd item and click OK.\n\tIf there is no 2nd (or 3rd) item for #2 or #3, click Cancel.",
+	sub {
+		my $expected_stdout = 'foo bar';
+		my $cmd1 = "$ZENITY --entry --entry-text 'foo bar'";
+		my $cmd2 = "$ZENITY --entry --entry-text 'baz boo' 'foo bar'";
+		my $cmd3 = "$ZENITY --entry --entry-text 'bleh bah' 'baz boo' 'foo bar'";
+
+		foreach ($cmd1, $cmd2, $cmd3) {
+			test_cmd_for_stdout ($_, $expected_stdout);
+		}
+	}
+);
+
+create_test ("test_calendar",
+	"Some calendars will appear. Follow the instructions on each dialog.",
+	sub {
+		my $iso_date = `date --iso-8601`;
+		chomp $iso_date;
+		my $base_cmd = "$ZENITY --calendar --date-format='%F'";
+
+		test_cmd_for_stdout (qq[$base_cmd --text="Ensure today's date is selected and click OK. If the date is wrong, click Cancel."], $iso_date);
+		test_cmd_for_stdout (qq[$base_cmd --text="Select November 11, 1985 and click OK."], '1985-11-11');
+		test_cmd_for_stdout (qq[$base_cmd --text="Click OK." --day=29 --month=2 --year=2024], '2024-02-29');
+	}
+);
+
 # MAIN
 
 if ($ARGV[0])
